# -*- coding: mbcs -*-
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Pore insert as a cavity in a FE model of a cylindrical volume 
# to simulate push-pull test 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Define Part Dimensions / Material & Mesh Properties
Radius_CylVol = 1.0 
Length_CylVol = 2.0

EModul_CylVol = 70000.0; EModul_Inclusion = 150000.0
PoissonsRatio_CylVol = 0.3; PoissonsRatio_Inclusion = 0.28
Density_CylVol = 2.7e-09
  
Seed_CylVol = 0.2;   minSizeValue_CylVol = 0.01
Seed_Inclusion   = 0.01 
 
InclusionName = 'Inclusion1'
NomStress = 10.0

 
# Import moduls
from abaqus import*
from abaqusConstants import*

import job 
import part
import step 
import load 
import mesh
import sketch 
import section 
import material 
import assembly
import numpy as np
import interaction 
import regionToolset

import math   

ModelCounter = 1

# Displacement
Disp = (NomStress*Length_CylVol)/EModul_CylVol

#-------------------------------------------------------------------------------------------------------------
# Import Inclusion
#-------------------------------------------------------------------------------------------------------------
mdb.openIges('Z:\LOEWE\Flaechenrueckfuehrung_CT-Scans'+'\\'+InclusionName+'.igs', 
    msbo=False, scaleFromFile=OFF, trimCurve=DEFAULT)
mdb.models['Model-1'].PartFromGeometryFile(combine=False, convertToAnalytical=1
    , dimensionality=THREE_D, geometryFile=mdb.acis, name='Inclusion', stitchEdges=1
    , stitchTolerance=1.0, type=DEFORMABLE_BODY)
mdb.models.changeKey(fromName='Model-1', toName=InclusionName+'_Rotation_Modelling')

#-------------------------------------------------------------------------------------------------------------
p = mdb.models[InclusionName+'_Rotation_Modelling']
pa = p.rootAssembly

  # Create Part 'Cylindrical_Volume'
p.ConstrainedSketch(name='__profile__', sheetSize=10.0)
p.sketches['__profile__'].CircleByCenterPerimeter(center=(0.0, 0.0), point1=(0.0, Radius_CylVol))
p.Part(dimensionality=THREE_D, name='Cylindrical_Volume', type=DEFORMABLE_BODY)    
p.parts['Cylindrical_Volume'].BaseSolidExtrude(depth=Length_CylVol, sketch=p.sketches['__profile__'])

  # Material CylVol / Inclusion
p.Material(name='AlSi9Cu3')
p.materials['AlSi9Cu3'].Density(table=((Density_CylVol, ), ))
p.materials['AlSi9Cu3'].Elastic(table=((EModul_CylVol, PoissonsRatio_CylVol), ))

p.Material(name='Inclusion')
p.materials['Inclusion'].Elastic(table=((EModul_Inclusion, PoissonsRatio_Inclusion), ))
   
  # Create Instances 
pa.Instance(dependent=OFF, name='Inclusion-1', part=p.parts['Inclusion'])                                                          
pa.rotate(angle=-10.0, axisDirection=(-1.0, 0.0, 0.0), axisPoint=(0.0, 0.0, 0.0), instanceList=('Inclusion-1', ))
pa.rotate(angle=-22.0, axisDirection=(0.0, -1.0, 0.0), axisPoint=(0.0, 0.0, 0.0), instanceList=('Inclusion-1', ))

pa.Instance(dependent=ON, name='Cylindrical_Volume-1', part=p.parts['Cylindrical_Volume'])
pa.translate(instanceList=('Cylindrical_Volume-1', ), vector=(0.0, 0.0, -Length_CylVol/2))
pa.makeDependent(instances=(pa.instances['Inclusion-1'], ))

#-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - 
# Merge Instances
pa.InstanceFromBooleanCut(cuttingInstances=(pa.instances['Inclusion-1'], ), instanceToBeCut=
    pa.instances['Cylindrical_Volume-1'], name='InclusionInCylVol', originalInstances=DELETE)

pp = p.parts['InclusionInCylVol']
pi = pa.instances['InclusionInCylVol-1']                                          
ppv, ppe, ppf, ppc = pp.vertices, pp.edges, pp.faces, pp.cells
piv, pie, pif, pic = pi.vertices, pi.edges, pi.faces, pi.cells

#-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
# Create Mesh  
pa.makeIndependent(instances=(pi, )) 

Edges_Inclusion = pie.getByBoundingSphere(center=(0.0, 0.0, 0.0), radius=0.75)
Faces_Inclusion = pif.getByBoundingSphere(center=(0.0,0.0,0.0), radius=0.75)
Cell_CylVol = (pic.findAt(((0.0, 0.0, Length_CylVol/2), ))+pic.findAt(((Radius_CylVol, 0.0, 0.0), ))+
    pic.findAt(((0.0, 0.1, -Length_CylVol/2), ))+pic.findAt(((0.0, -0.1, -Length_CylVol/2), )))
Cell_Model = pic.getByBoundingCylinder(center1=(0.0,0.0,-(Length_CylVol/2)-0.1), 
    center2=(0.0,0.0,(Length_CylVol/2)+0.1), radius=Radius_CylVol+0.1) 

pa.setMeshControls(allowMapped=True, elemShape=TET, regions=Cell_CylVol, technique=FREE)
pa.setMeshControls(allowMapped=False, elemShape=TET, regions=Faces_Inclusion, technique=FREE)

pa.setElementType(elemTypes=(mesh.ElemType(elemCode=C3D20R, elemLibrary=STANDARD), 
    mesh.ElemType(elemCode=C3D15, elemLibrary=STANDARD), mesh.ElemType(elemCode=C3D10, 
    elemLibrary=STANDARD)), regions=(Cell_Model, ))

pa.seedPartInstance(regions=(pi, ), deviationFactor=0.1, minSizeValue=minSizeValue_CylVol, size=Seed_CylVol)
pa.seedEdgeBySize(constraint=FINER, deviationFactor=0.1, edges=Edges_Inclusion, minSizeFactor=0.1, size=Seed_Inclusion)

pa.generateMesh(regions=(pi, ))

#-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - 
# Partition Bottom     
Edge_Bottom=ppe.findAt((Radius_CylVol,0.0,-(Length_CylVol/2.0)), )
pp.PartitionEdgeByParam(edges=Edge_Bottom, parameter=0.5)

Face_Bottom=ppf.getByBoundingCylinder(center1=(0.0,0.0,-(Length_CylVol/2.)-0.1), 
    center2=(0.0,0.0,-(Length_CylVol/2.)+0.1), radius=Radius_CylVol+0.1)
pp.PartitionFaceByShortestPath(faces=Face_Bottom, point1=pp.InterestingPoint(edge=
    ppe.findAt(coordinates=(-Radius_CylVol, 0, -Length_CylVol/2)), rule=MIDDLE), point2=
    pp.InterestingPoint(edge=ppe.findAt(coordinates=(Radius_CylVol, 0, -Length_CylVol/2)), rule=MIDDLE))
    
Edges_Partition = ppe.findAt(((0.0, 0.0, -Length_CylVol/2), ))
pp.PartitionEdgeByParam(edges=Edges_Partition, parameter=0.5)

pa.regenerate()

#-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
# Sets / Surfaces 
Face_Top = pif.getByBoundingCylinder(center1=(0.0,0.0,(Length_CylVol/2)-0.1), center2=
    (0.0,0.0,(Length_CylVol/2)+0.1), radius=Radius_CylVol+0.1)
pa.Set(faces=Face_Top, name='Top')

RefPoint = piv.getByBoundingSphere(center=(0.0,0.0,-(Length_CylVol/2)), radius=0.05)
pa.Set(name='RefPoint', vertices= RefPoint)

Face_Bottom = pif.getByBoundingCylinder(center1=(0.0,0.0,-(Length_CylVol/2.)-0.1), 
    center2=(0.0,0.0,-(Length_CylVol/2.)+0.1), radius=Radius_CylVol+0.1)
pa.Set(faces=Face_Bottom, name='Bottom')

pa.Surface(name='Inclusion', side1Faces=Faces_Inclusion)  

#-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
# Step / BC / Load  
p.StaticStep(name='Pull', previous='Initial')
p.DisplacementBC(amplitude=UNSET, createStepName=
    'Initial', distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'FixBottom', region=pa.sets['Bottom']
    , u1=UNSET, u2=UNSET, u3=SET, ur1=UNSET, ur2=UNSET, ur3=UNSET)
p.EncastreBC(createStepName='Initial', localCsys=None, 
    name='RefPoint', region=pa.sets['RefPoint'])

p.DisplacementBC(amplitude=UNSET, createStepName=
    'Pull', distributionType=UNIFORM, fieldName='', fixed=OFF, localCsys=
    None, name='Pull_Glob', region=pa.sets['Top'], 
    u1=UNSET, u2=UNSET, u3=Disp, ur1=UNSET, ur2=UNSET, ur3=UNSET)


